"""Mapping between icon numbers and their string representations"""

ICONS = {1: {'weather_id': 'not_available', 'weather': 'Not available'},
         2: {'weather_id': 'sunny', 'weather': 'Sunny'},
         3: {'weather_id': 'mostly_sunny', 'weather': 'Mostly sunny'},
         4: {'weather_id': 'partly_sunny', 'weather': 'Partly sunny'},
         5: {'weather_id': 'mostly_cloudy', 'weather': 'Mostly cloudy'},
         6: {'weather_id': 'cloudy', 'weather': 'Cloudy'},
         7: {'weather_id': 'overcast', 'weather': 'Overcast'},
         8: {'weather_id': 'overcast_w_low_clds', 'weather': 'Overcast with low clouds'},
         9: {'weather_id': 'fog', 'weather': 'Fog'},
         10: {'weather_id': 'light_rain', 'weather': 'Light rain'},
         11: {'weather_id': 'rain', 'weather': 'Rain'},
         12: {'weather_id': 'psbl_rain', 'weather': 'Possible rain'},
         13: {'weather_id': 'rain_shower', 'weather': 'Rain shower'},
         14: {'weather_id': 'tstorm', 'weather': 'Thunderstorm'},
         15: {'weather_id': 'tstorm_shower', 'weather': 'Local thunderstorms'},
         16: {'weather_id': 'light_snow', 'weather': 'Light snow'},
         17: {'weather_id': 'snow', 'weather': 'Snow'},
         18: {'weather_id': 'psbl_snow', 'weather': 'Possible snow'},
         19: {'weather_id': 'snow_shower', 'weather': 'Snow shower'},
         20: {'weather_id': 'rain_and_snow', 'weather': 'Rain and snow'},
         21: {'weather_id': 'psbl_rain_and_snow', 'weather': 'Possible rain and snow'},
         22: {'weather_id': 'rain_and_snow', 'weather': 'Rain and snow'},
         23: {'weather_id': 'fr_rain', 'weather': 'Freezing rain'},
         24: {'weather_id': 'psbl_fr_rain', 'weather': 'Possible freezing rain'},
         25: {'weather_id': 'hail', 'weather': 'Hail'},
         26: {'weather_id': 'clear', 'weather': 'Clear'},
         27: {'weather_id': 'mostly_clear', 'weather': 'Mostly clear'},
         28: {'weather_id': 'partly_clear', 'weather': 'Partly clear'},
         29: {'weather_id': 'mostly_cloudy', 'weather': 'Mostly cloudy'},
         30: {'weather_id': 'cloudy', 'weather': 'Cloudy'},
         31: {'weather_id': 'overcast_w_low_clds', 'weather': 'Overcast with low clouds'},
         32: {'weather_id': 'rain_shower', 'weather': 'Rain shower'},
         33: {'weather_id': 'tstorm_shower', 'weather': 'Local thunderstorms'},
         34: {'weather_id': 'snow_shower', 'weather': 'Snow shower'},
         35: {'weather_id': 'rain_and_snow', 'weather': 'Rain and snow'},
         36: {'weather_id': 'psbl_fr_rain', 'weather': 'Possible freezing rain'},
}



"""Tests for pymeteosource"""

import os
import sys
from datetime import datetime, date
from unittest.mock import MagicMock
from os.path import realpath, join, dirname
import pytz
import pytest
import pandas

from pymeteosource.api import Meteosource
from pymeteosource.types import tiers, endpoints, units, sections
from pymeteosource.types.time_formats import F1
from pymeteosource.data import (Forecast, SingleTimeData, MultipleTimesData,
                                AlertsData)
from pymeteosource.errors import (InvalidArgumentError, InvalidIndexTypeError,
                                  InvalidStrIndexError, EmptyInstanceError,
                                  InvalidDatetimeIndexError, InvalidDateFormat,
                                  InvalidDateSpecification, InvalidDateRange)

from .sample_data import SAMPLE_POINT, SAMPLE_TIME_MACHINE
from .dst_changes_data import LONG_DAY
from .variables_list import (CURRENT, PRECIPITATION_CURRENT, WIND, MINUTELY,
                             HOURLY, CLOUD, PRECIPITATION, PROBABILITY, DAILY,
                             ALL_DAY, PART_DAY, ASTRO, SUN, MOON, STATS,
                             STATS_TEMP, STATS_WIND, STATS_PREC, ALERTS)

sys.path.insert(0, realpath(join(dirname(__file__), "..")))

# Load API key from environment variable
API_KEY = os.environ.get('METEOSOURCE_API_KEY')
if API_KEY is None:
    raise ValueError("You need to provide API key as environment variable.")


def test_to_dst_changes():
    """Test exporting to pandas"""
    m = Meteosource(API_KEY, tiers.FLEXI)
    # We mock the API requests with sample data
    m.req_handler.execute_request = MagicMock(return_value=LONG_DAY)
    # Get the mocked forecast
    f = m.get_point_forecast(place_id='london', tz='Europe/Prague')
    # Check the ambiguous date is handled properly
    dts = ['2021-10-31T01:00:00', '2021-10-31T02:00:00',
           '2021-10-31T02:00:00', '2021-10-31T03:00:00']
    assert f.hourly.dates_str == dts

    # Check the astro datetimes are OK when DST changes
    tz = pytz.timezone('Europe/Prague')
    dt = tz.localize(datetime(2021, 10, 31, 1, 24, 35), is_dst=None)
    assert f.daily[0].astro.sun.rise == dt
    dt = tz.localize(datetime(2021, 10, 31, 2, 24, 35), is_dst=True)
    assert f.daily[0].astro.sun.set == dt
    dt = tz.localize(datetime(2021, 10, 31, 2, 24, 35), is_dst=False)
    assert f.daily[0].astro.moon.rise == dt
    dt = tz.localize(datetime(2021, 10, 31, 3, 24, 35), is_dst=None)
    assert f.daily[0].astro.moon.set == dt


def test_build_url():
    """Test URL building"""
    url = 'https://www.meteosource.com/api/v1/%s/%s'
    for tier in [tiers.FLEXI, tiers.STANDARD, tiers.STARTUP, tiers.FREE]:
        for endpoint in [endpoints.POINT, endpoints.TIME_MACHINE]:
            m = Meteosource(API_KEY, tier)
            assert m._build_url(endpoint) == url % (tier, endpoint)


def test_get_point_forecast_exceptions():
    """Test detection of invalid point specification detection"""
    m = Meteosource(API_KEY, tiers.FLEXI)
    # We mock the API requests with sample data
    m.req_handler.execute_request = MagicMock(return_value=SAMPLE_POINT)

    # Test invalid place definitions
    with pytest.raises(InvalidArgumentError) as e:
        m.get_point_forecast(place_id='london', lat=50)
    assert str(e.value) == 'Only place_id or lat+lon can be specified!'
    with pytest.raises(InvalidArgumentError) as e:
        m.get_point_forecast(place_id='london', lon=14)
    assert str(e.value) == 'Only place_id or lat+lon can be specified!'
    with pytest.raises(InvalidArgumentError) as e:
        m.get_point_forecast(place_id='london', lat=50, lon=14)
    assert str(e.value) == 'Only place_id or lat+lon can be specified!'
    with pytest.raises(InvalidArgumentError) as e:
        m.get_point_forecast(lat=50)
    assert str(e.value) == 'Only place_id or lat+lon can be specified!'
    with pytest.raises(InvalidArgumentError) as e:
        m.get_point_forecast(lon=14)
    assert str(e.value) == 'Only place_id or lat+lon can be specified!'
    with pytest.raises(InvalidArgumentError) as e:
        m.get_point_forecast()
    assert str(e.value) == 'Only place_id or lat+lon can be specified!'

    # Test valid place definitions
    m.get_point_forecast(place_id='london')
    m.get_point_forecast(lat=50, lon=14)


def test_get_time_machine_exceptions():
    """Test date specification for get_time_machine"""
    m = Meteosource(API_KEY, tiers.FLEXI)
    # We mock the API requests with sample data
    m.req_handler.execute_request = MagicMock(return_value=SAMPLE_TIME_MACHINE)

    # Test invalid dates
    with pytest.raises(InvalidDateFormat) as e:
        m.get_time_machine(date='2021-01-0', place_id='london')
    assert str(e.value) == 'Invalid date "2021-01-0", should be "%Y-%m-%d"'
    with pytest.raises(InvalidDateFormat) as e:
        m.get_time_machine(date='fgh', place_id='london')
    assert str(e.value) == 'Invalid date "fgh", should be "%Y-%m-%d"'
    with pytest.raises(InvalidDateFormat) as e:
        m.get_time_machine(date=5, place_id='london')
    assert 'str or date instance' in str(e.value)

    # Test invalid date specifications
    with pytest.raises(InvalidDateSpecification) as e:
        m.get_time_machine(place_id='london')
    assert str(e.value) == 'Specify either "date" or "date_from" and "date_to"'
    with pytest.raises(InvalidDateSpecification) as e:
        m.get_time_machine(date='2021-01-01', date_to='2021-01-02',
                           place_id='london')
    assert str(e.value) == 'Specify either "date" or "date_from" and "date_to"'
    with pytest.raises(InvalidDateSpecification) as e:
        m.get_time_machine(date='2021-01-01', date_from='2021-01-02',
                           place_id='london')
    assert str(e.value) == 'Specify either "date" or "date_from" and "date_to"'
    with pytest.raises(InvalidDateSpecification) as e:
        m.get_time_machine(date_to='2021-01-02', place_id='london')
    assert str(e.value) == 'Specify either "date" or "date_from" and "date_to"'
    with pytest.raises(InvalidDateSpecification) as e:
        m.get_time_machine(date_from='2021-01-02', place_id='london')
    assert str(e.value) == 'Specify either "date" or "date_from" and "date_to"'
    with pytest.raises(InvalidDateSpecification) as e:
        m.get_time_machine(date='2021-01-01', date_from='2021-01-01',
                           date_to='2021-01-02', place_id='london')
    assert str(e.value) == 'Specify either "date" or "date_from" and "date_to"'

    # Test invalid date range
    with pytest.raises(InvalidDateRange) as e:
        m.get_time_machine(date_from='2021-01-03',
                           date_to=datetime(2021, 1, 3, 23, 59),
                           place_id='london')
    assert 'is not smaller than "date_to"' in str(e.value)

    # Test valid date definitions
    m.get_time_machine(date='2021-01-01', place_id='london')
    m.get_time_machine(date=['2021-01-01', datetime(2021, 1, 2)],
                       place_id='london')
    m.get_time_machine(date_from='2021-01-01', date_to=datetime(2021, 1, 3),
                       place_id='london')
    m.get_time_machine(date_from='2021-01-01', date_to=date(2021, 1, 3),
                       place_id='london')


def test_forecast_indexing():
    """Test indexing MultipleTimesData with int, string and datetimes"""
    m = Meteosource(API_KEY, tiers.FLEXI)
    # We mock the API requests with sample data
    m.req_handler.execute_request = MagicMock(return_value=SAMPLE_POINT)
    # Get the mocked forecast
    f = m.get_point_forecast(place_id='london', tz='UTC')

    # Index by int
    assert f.hourly[1].wind.angle == 106

    # Index by too large int
    with pytest.raises(IndexError):
        assert f.hourly[1000]

    # Index by string
    assert f.hourly['2021-09-08T11:00:00'].feels_like == 23.2

    # Index by string with wrong format
    with pytest.raises(InvalidStrIndexError):
        f.hourly['2021-09-08 11:00:00']  # pylint: disable=W0104

    # Index by unsupported type
    with pytest.raises(InvalidIndexTypeError):
        f.hourly[1.1]  # pylint: disable=W0104

    # Index by tz-naive datetime
    dt = datetime.strptime('2021-09-09T00:00:00', F1)
    assert f.hourly[dt].probability.precipitation == 61

    # Index by tz-aware datetime but in different tz
    dt1 = pytz.timezone('UTC').localize(dt)
    assert f.hourly[dt1].probability.precipitation == 61

    # Index by tz-aware datetime
    dt1 = pytz.timezone('Europe/London').localize(dt)
    assert f.hourly[dt1].probability.precipitation == 21

    # Index by tz-aware datetime but with wrong timezone
    dt2 = pytz.timezone('Asia/Kabul').localize(dt)
    with pytest.raises(InvalidDatetimeIndexError) as e:
        f.hourly[dt2]  # pylint: disable=W0104
    assert 'Invalid datetime index' in str(e.value)

    # Check timezone settings
    f = m.get_point_forecast(place_id='london', tz='Europe/London')
    dt = pytz.timezone('Europe/London').localize(datetime(2021, 9, 8, 10))
    assert f.hourly[0].date == dt

    f = m.get_point_forecast(place_id='london', tz='Europe/Prague')
    dt = pytz.timezone('Europe/Prague').localize(datetime(2021, 9, 8, 11))
    assert f.hourly[0].date == dt

    f = m.get_point_forecast(place_id='london', tz='Asia/Kabul')
    dt = pytz.timezone('Asia/Kabul').localize(datetime(2021, 9, 8, 13, 30))
    assert f.hourly[0].date == dt


def test_to_pandas():
    """Test exporting to pandas"""
    m = Meteosource(API_KEY, tiers.FLEXI)
    # We mock the API requests with sample data
    m.req_handler.execute_request = MagicMock(return_value=SAMPLE_POINT)
    # Get the mocked forecast
    f = m.get_point_forecast(place_id='london')

    df = f.current.to_pandas()
    assert len(df) == 1

    df = f.minutely.to_pandas()
    assert len(df) == 116
    assert isinstance(df.index, pandas.core.indexes.datetimes.DatetimeIndex)

    df = f.hourly.to_pandas()
    assert len(df) == 155
    assert isinstance(df.index, pandas.core.indexes.datetimes.DatetimeIndex)

    df = f.daily.to_pandas()
    assert len(df) == 30
    assert isinstance(df.index, pandas.core.indexes.datetimes.DatetimeIndex)

    df = f.alerts.to_pandas()
    assert len(df) == 4


def test_to_dict():
    """Test exporting to pandas"""
    m = Meteosource(API_KEY, tiers.FLEXI)
    # We mock the API requests with sample data
    m.req_handler.execute_request = MagicMock(return_value=SAMPLE_POINT)
    # Get the mocked forecast
    f = m.get_point_forecast(place_id='london')

    # Test multilevel dict flattening
    assert 'afternoon_wind_angle' in f.daily[0].to_dict()


def test_forecast_structure():
    """Test structure of the Forecast object on real data"""
    # Initialize the Meteosource object
    m = Meteosource(API_KEY, tiers.FLEXI)
    # Get real forecast data (not mocked)
    f = m.get_point_forecast(place_id='london', tz='Asia/Kabul',
                             units=units.METRIC, sections=sections.ALL)

    # Check if the header is correct
    assert isinstance(f, Forecast)
    assert f.lat == 51.50853
    assert f.lon == -0.12574
    assert f.elevation == 25
    assert f.timezone == 'Asia/Kabul'
    assert f.units == 'metric'

    # Check types of the sections
    assert isinstance(f.current, SingleTimeData)
    assert isinstance(f.minutely, MultipleTimesData)
    assert isinstance(f.hourly, MultipleTimesData)
    assert isinstance(f.daily, MultipleTimesData)

    # Check current section
    assert set(f.current.get_members()) == CURRENT
    assert set(f.current.wind.get_members()) == WIND  # pylint: disable=E1101
    # pylint: disable=E1101
    assert set(f.current.precipitation.get_members()) == PRECIPITATION_CURRENT

    # Check minutely section
    assert isinstance(f.minutely.summary, str)
    assert isinstance(f.minutely, MultipleTimesData)
    assert len(f.minutely) > 0
    assert set(f.minutely[0].get_members()) == MINUTELY
    assert isinstance(f.minutely[0].date, datetime)

    # Check hourly section
    assert isinstance(f.hourly, MultipleTimesData)
    assert len(f.hourly) > 0
    assert set(f.hourly[0].get_members()) == HOURLY
    assert set(f.hourly[0].wind.get_members()) == WIND
    assert set(f.hourly[0].cloud_cover.get_members()) == CLOUD
    assert set(f.hourly[0].precipitation.get_members()) == PRECIPITATION
    assert set(f.hourly[0].probability.get_members()) == PROBABILITY

    # Check daily section
    assert isinstance(f.daily, MultipleTimesData)
    assert set(f.daily[0].get_members()) == DAILY
    assert isinstance(f.daily[0].day, datetime)
    assert set(f.daily[0].all_day.get_members()) == ALL_DAY
    assert set(f.daily[0].all_day.wind.get_members()) == WIND
    assert set(f.daily[0].morning.get_members()) == PART_DAY
    assert set(f.daily[0].morning.wind.get_members()) == WIND
    assert set(f.daily[0].afternoon.get_members()) == PART_DAY
    assert set(f.daily[0].afternoon.wind.get_members()) == WIND
    assert set(f.daily[0].evening.get_members()) == PART_DAY
    assert set(f.daily[0].evening.wind.get_members()) == WIND
    assert set(f.daily[0].astro.get_members()) == ASTRO
    assert set(f.daily[0].astro.sun.get_members()) == SUN
    assert set(f.daily[0].astro.moon.get_members()) == MOON
    assert set(f.daily[0].statistics.get_members()) == STATS
    assert set(f.daily[0].statistics.temperature.get_members()) == STATS_TEMP
    assert set(f.daily[0].statistics.wind.get_members()) == STATS_WIND
    assert set(f.daily[0].statistics.precipitation.get_members()) == STATS_PREC

    # Check alerts section
    assert isinstance(f.alerts, AlertsData)

    # Check correct exception raising when minutely data are not present
    m = Meteosource(API_KEY, tiers.FREE)
    # Get real forecast data (not mocked)
    f = m.get_point_forecast(place_id='london')
    with pytest.raises(EmptyInstanceError) as e:
        f.minutely[0]  # pylint: disable=W0104
    assert str(e.value) == 'The instance does not contain any data!'


def test_time_machine_structure():
    """Test structure of the Forecast object on real data"""
    # Shortcut for UTC timezone object
    utc = pytz.timezone('UTC')
    # Shortcut for Kabul timezone object
    kbl = pytz.timezone('US/Pacific')

    # Initialize the Meteosource object
    m = Meteosource(API_KEY, tiers.FLEXI)
    # Get real forecast data (not mocked)
    tm = m.get_time_machine(date=[date(2021, 1, 1), '2019-05-05',
                            datetime(2020, 12, 15, 1, 10, 25)],
                            place_id='london', tz='UTC')
    assert len(tm.data) == 72
    assert tm.data[0].date == datetime(2021, 1, 1, 0, tzinfo=utc)
    assert tm.data[-1].date == datetime(2020, 12, 15, 23, tzinfo=utc)
    assert tm.data[25].date == datetime(2019, 5, 5, 1, tzinfo=utc)

    tm = m.get_time_machine(date_from='2019-05-05', date_to=date(2019, 5, 9),
                            lat=35.295, lon=69.5, tz='Asia/Kabul')
    assert len(tm.data) == 120
    dt = pytz.utc.localize(datetime(2019, 5, 5)).astimezone(kbl)
    assert tm.data[0].date == dt


def test_alerts():
    """Test alerts"""
    m = Meteosource(API_KEY, tiers.FLEXI)
    # We mock the API requests with sample data
    m.req_handler.execute_request = MagicMock(return_value=SAMPLE_POINT)
    # Get the mocked alerts data
    alerts = m.get_point_forecast(place_id='london', tz='UTC').alerts

    assert len(alerts) == 4
    for a in alerts:
        assert set(a.get_members()) == ALERTS

    assert alerts[3].event == 'Moderate Thunderstorms'
    assert len(alerts.get_active_alerts('2022-03-08T22:10:00')) == 3
    assert len(alerts.get_active_alerts(datetime(2022, 3, 8, 23, 0, 0))) == 3

    dt = pytz.timezone("Asia/Bangkok").localize(datetime(2022, 3, 8, 23, 0, 0))
    assert len(alerts.get_active_alerts(dt)) == 2



"""Module that provide the Meteosource interface object"""

import datetime as dt

from .request_handler import RequestHandler
from .types import langs, sections, units, endpoints, time_formats
from .errors import (InvalidArgumentError, InvalidDateFormat, InvalidDateRange,
                     InvalidDateSpecification)
from .data import Forecast, TimeMachine


class Meteosource:
    """
    The main object that provides the interface for Meteosource API

    Attributes
    ----------
    req_handler : RequestHandler
        RequestHandler object to be used for the requests
    host : string
        The host URL of the Meteosource API
    tier : string
        The tier the user is using

    Methods
    -------
    _build_url
        Build URL for the request
    get_point_forecast
        Get forecast data for given point
    """
    def __init__(self, api_key, tier, host='https://www.meteosource.com/api',
                 use_gzip=True):
        """
        Basic constructor

        :param str: API key
        :param str: Tier the user is using
        :param str: Host URL of the Meteosource API
        :param bool: True if gzip compression should be used, False otherwise
        """
        # Initialize the request handler with the API key
        self.req_handler = RequestHandler(api_key, use_gzip)
        self.host = host
        self.tier = tier

    def _build_url(self, endpoint):
        """
        Build URL for the request

        :param str: Endpoint for the request
        :return str: The URL of the request without parameters (lat, lon, ...)
        """
        pars = {'host': self.host, 'tier': self.tier, 'endpoint': endpoint}
        url = '{host}/v1/{tier}/{endpoint}'.format(**pars)

        return url

    # pylint: disable=R0201
    def _build_location_pars(self, pars, place_id, lat, lon):
        """
        Add location specification to parameters dict

        The request should contain either place_id or lat+lon. This function
        checks whether the location is fully and non-ambiguously specified,
        and adds proper parameters.

        :param dict: Dictionary with the other-than-location parameter(s)
        :param str: Identifier of the place (place_id)
        :param float: Latitude of the point
        :param float: Longitude of the point
        :return dict: Dictionary with location parameter(s) set
        """
        # If place_id is not specified, we use lat+lon
        if place_id is None:
            # Check if both lat+lon are specified
            if lat is None or lon is None:
                raise InvalidArgumentError()
            # Add the lat+lon to the parameters
            pars.update({'lat': lat, 'lon': lon})
        else:
            # Check that only place_id is specified
            if lat is not None or lon is not None:
                raise InvalidArgumentError()
            # Add the place_id to the parameters
            pars.update({'place_id': place_id})

        return pars

    def get_point_forecast(self, place_id=None, lat=None, lon=None,
                           sections=(sections.CURRENT, sections.HOURLY),
                           tz='UTC', lang=langs.ENGLISH, units=units.AUTO):
        """
        Get forecast data for given point

        :param str: Identifier of the place (place_id)
        :param float: Latitude of the point
        :param float: Longitude of the point
        :param str: Sections to return
        :param str: Timezone for final output. Requests are always made in UTC!
        :param str: Language
        :param str: Units to use
        :return Forecast: Forecast object with the forecast data
        """
        # Build the URL for the request
        url = self._build_url(endpoints.POINT)
        if isinstance(sections, (list, tuple)):
            sections = ','.join(sections)
        # Parameters of the request, the requested tz is always UTC!
        pars = {'language': lang, 'units': units, 'timezone': 'UTC',
                'sections': sections}

        # Update parameters with location selection
        pars = self._build_location_pars(pars, place_id, lat, lon)

        # Execute the request with the built URL and parameters
        data = self.req_handler.execute_request(url, **pars)

        # Load the result into Forecast object and return it
        return Forecast(data, tz)

    def _str_to_date(self, date):
        """
        Convert passed date to datetime.date instance

        :param str/datetime.date/datetime.datetime: Passed date
        :return datetime.date: Date instance
        """
        if type(date) is dt.date:  # pylint: disable=C0123
            return date
        if type(date) is dt.datetime:  # pylint: disable=C0123
            return date.date()

        # Convert string date to datetime.date, while checking the format
        try:
            return dt.datetime.strptime(date, time_formats.F2).date()
        except ValueError as e:
            raise InvalidDateFormat(date, time_formats.F2) from e

    def _get_tm_dates(self, date, date_from, date_to):
        """
        Get date list for the time_machine endpoint

        The user can either specify 'date', which can be string, datetime.date,
        datetime.datetime or list/set/tuple of these types; or date range
        (by specifying date_from, date_to - both are inclusive). We first check
        whether only one of these options were used, and then convert it to
        list of datetime.date instances, that we want to retrieve.

        :param iterable/str/datetime.date/datetime.datetime: Passed date(s)
        :param str/datetime.date/datetime.datetime: Start point of date range
        :param str/datetime.date/datetime.datetime: End point of date range
        """
        # Check date specification validity
        if date is None and (date_from is None or date_to is None):
            raise InvalidDateSpecification()
        if date is not None and (date_from is not None or date_to is not None):
            raise InvalidDateSpecification()

        # If we are not using range
        if date is not None:
            # Convert passed date(s) to list, if not allowed iterable
            return date if isinstance(date, (list, tuple, set)) else [date]

        # Convert the start and end dates to datetime.date
        dt_from = self._str_to_date(date_from)
        dt_to = self._str_to_date(date_to)

        # If we are using range, check date_from is smaller than date_to
        if dt_to <= dt_from:
            raise InvalidDateRange(date_from, date_to)

        # Calculate number of days from dt_from we need
        no_days = (dt_to - dt_from).days + 1

        # Return a list with these dates
        return [dt_from + dt.timedelta(days=x) for x in range(no_days)]

    def get_time_machine(self, date=None, date_from=None, date_to=None,
                         place_id=None, lat=None, lon=None, tz='UTC',
                         units=units.AUTO):
        """
        Get archive data from time_machine endpoint

        The user can either specify 'date', which can be string, datetime.date,
        datetime.datetime or list/set/tuple of these types; or date range
        (by specifying date_from, date_to - both are inclusive).

        :param str/date/iterable/None: Date(s) to retrieve
        :param str/date/None: Starting date (inclusive)
        :param str/date/None: End date (inclusive)
        :param str: Identifier of the place (place_id)
        :param float: Latitude of the point
        :param float: Longitude of the point
        :param str: Timezone for final output. Requests are always made in UTC!
        :param str: Units to use
        :return TimeMachine: TimeMachine object with the archive data
        """
        # Build the URL for the request
        url = self._build_url(endpoints.TIME_MACHINE)

        # Update parameters with location selection, date will be added in loop
        pars = self._build_location_pars({'units': units}, place_id, lat, lon)
        # We need to specify UTC timezone
        pars['timezone'] = 'UTC'

        # Get list of dates to retrieve
        dates = self._get_tm_dates(date, date_from, date_to)

        # Placeholder for the resulting TimeMachine object
        tm = None
        # Iterate over the dates
        for d in dates:
            # Convert date to proper string format
            if type(d) is dt.date:  # pylint: disable=C0123
                pars['date'] = d.strftime(time_formats.F2)
            elif type(d) is dt.datetime:  # pylint: disable=C0123
                pars['date'] = d.date().strftime(time_formats.F2)
            elif isinstance(d, str):
                # Check the date format is correct for string dates
                _ = self._str_to_date(d)
                # Update the date in the parameters
                pars['date'] = d
            else:
                raise InvalidDateFormat(d, 'str or date instance')

            # Execute the request with the built URL and parameters
            data = self.req_handler.execute_request(url, **pars)

            # Create a TimeMachine instance
            cur_tm = TimeMachine(data, tz, d)

            # Assign the current instance to the result, or append it
            if tm is None:
                tm = cur_tm
            else:
                tm.append(cur_tm)

        return tm



from pathlib import Path
from setuptools import find_packages, setup


setup(
    version="1.5.0",
    name="pymeteosource",
    packages=find_packages(),
    install_requires=["wheel", "requests", "pytz"],
    extras_require={"pandas": "pandas"},
    description="Meteosource API wrapper library",
    long_description=Path("README.md").read_text(encoding="utf-8"),
    long_description_content_type='text/markdown',
    author="Meteosource",
    author_email="support@meteosource.com",
    url="https://github.com/Meteosource/pymeteosource/archive/v1.5.0.tar.gz",
    license="MIT",
    classifiers=[
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3.6",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)
